
# -*- coding:utf-8 -*-

import os
import re
import sys
import pandas as pd
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Blast import NCBIXML

# from Bio.Blast import NCBIWWW
from Bio.Blast.Applications import NcbiblastnCommandline
from Bio.Blast.Applications import NcbimakeblastdbCommandline


class BlastDbError(Exception):
    pass


class campycps():
    def __init__(self, inputfile, database, output, threads, minid=95, mincov=90):
        self.inputfile = os.path.abspath(inputfile)
        self.database = database
        self.minid = int(minid)
        self.mincov = int(mincov)
        self.temp_output = os.path.join(os.path.abspath(output), 'temp.txt')
        self.threads = threads

    def biopython_blast(self):
        # cline = NcbiblastnCommandline(query=self.inputfile, db=self.database, dust='no', ungapped=True,
        #                               evalue=1E-20, out=self.temp_output,  # delete culling_limit parameters
        #                               outfmt="6 sseqid slen length nident",
        #                               task='dc-megablast',
        #                               perc_identity=self.minid, max_target_seqs=1000000,
        #                               num_threads=self.threads)
        cline = NcbiblastnCommandline(query=self.inputfile, db=self.database, dust='no', ungapped=True,
                                      evalue=1E-20, out=self.temp_output,  # delete culling_limit parameters
                                      outfmt='"6 sseqid slen length nident"',
                                      # task='dc-megablast',
                                      perc_identity=self.minid, max_target_seqs=1000000,
                                      num_threads=self.threads)
        # print(cline)
        stdout, stderr = cline()
        df = pd.read_csv(self.temp_output, sep='\t', names=[
            'sseqid', 'slen', 'length', 'nident'])
        # print(df)

        result = {}
        for i, row in df.iterrows():
            gene, num = re.match(
                '^(\w+)[_-](\d+)', row['sseqid']).group(1, 2)
            # print(gene)
            num = int(num)
            hlen = row['slen']
            alen = row['length']
            nident = row['nident']
            if nident * 100 / hlen >= self.mincov:
                # if sch not in result.keys():  # check if sch is the key of result
                #     result[sch] = {}
                # resolve the bug that could not get exactly matched allele
                if hlen == alen & nident == hlen:  # exact match
                    if gene in result.keys():

                        if not re.search(r'[~\?]', str(result[gene])):
                            old_num = int(result[gene])
                            if num < old_num:
                                # print(f'{num}\t{old_num}')
                                result[gene] = num
                                print(
                                    f'Found additional allele match, replace {gene}:{old_num} -> {num}')
                            else:
                                print(
                                    f'Found additional allele match, but the allele number {num} is greater or equal to stored one {gene}:{old_num}, skip...')
                        else:  # replace not perfect match
                            result[gene] = num
                    else:
                        result[gene] = num
                # new allele
                elif (alen == hlen) & (nident != hlen):
                    # print('xx')
                    if gene not in result.keys():
                        # print('xxx')
                        result[gene] = f'~{num}'
                    else:
                        next
                elif (alen != hlen) & (nident == hlen):  # partial match
                    # print('xxxx')
                    if gene not in result.keys():
                        result[gene] = f'{num}?'
                else:
                    next
        # remove temp blastn output file
        os.remove(self.temp_output)
        return result

    @staticmethod
    def makeblastdb(file):
        cline = NcbimakeblastdbCommandline(
            dbtype="nucl", title='campycps',
            hash_index=True,
            parse_seqids=True,
            input_file=file)
        print(f"Making database...")
        stdout, stderr = cline()
        print(stdout)

    @staticmethod
    def is_fasta(file):
        """
        chcek if the input file is fasta format
        """
        try:
            with open(file, "r") as handle:
                fasta = SeqIO.parse(handle, "fasta")
                # False when `fasta` is empty, i.e. wasn't a FASTA file
                return any(fasta)
        except:
            return False



#!/usr/bin/env python3
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: Jake Cui
# Mail: cqp@cau.edu.cn
# Created Time:  2024-1-10 14:49:41
#############################################

import os
import sys

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup


requirements = [
    'Bio', 'pandas', 'setuptools'
]


def read(rel_path: str) -> str:
    here = os.path.abspath(os.path.dirname(__file__))
    # intentionally *not* adding an encoding option to open, See:
    #   https://github.com/pypa/virtualenv/issues/201#issuecomment-3145690
    with open(os.path.join(here, rel_path)) as fp:
        return fp.read()


def get_version(rel_path: str) -> str:
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")


about = {}
here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'CampyCPS', '__init__.py'), 'r', encoding='utf-8') as f:
    exec(f.read(), about)


# Get the long description from the relevant file
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name="CampyCPS",
    version=about['__version__'],
    keywords=["wgs", "Campylobacter CPS"],
    description="Campylobacter CPS relevant gene analysis tool",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="MIT Licence",
    url="https://github.com/hbucqp/CampyCPS",
    author=about['__author__'],
    author_email=about['__author_email__'],
    packages=find_packages(),
    include_package_data=True,
    # package_data={'': ['*']},
    platforms="any",
    install_requires=requirements,
    classifiers=[
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Development Status :: 3 - Alpha',
        # Define that your audience are developers
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   # Again, pick a license
        # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    entry_points={
        'console_scripts': [
            'CampyCPS=CampyCPS.CampyCPS:main',
        ],
    },
)
